#!/usr/bin/perl -w
#
# shinGETsu - P2P anonymous BBS
#
# (c) Fuktommy <fuktommy@users.sourceforge.net>
# Released under the GNU General Public License
# $Id: gateway.cgi,v 1.101 2004/12/21 04:47:59 fuktommy Exp $
#

use strict;
use lib qw(..);
use Shingetsu::Config;
use Shingetsu::NodeList;
use Shingetsu::Cache;
use Shingetsu::CacheStat;
use Shingetsu::Gateway;
use Shingetsu::Message;
use Shingetsu::Signature;
use Shingetsu::Util;

umask $Shingetsu::Config::umask;

my $datadir	= $Shingetsu::Config::datadir;
my $root	= $Shingetsu::Config::root;
my @types	= qw(list thread note);
my $message	= $Shingetsu::Gateway::message;

#
# print title page
#
sub printTitle() {
	if ($ENV{REMOTE_ADDR} =~ $Shingetsu::Config::adminAddr) {
		printHeader($$message{logo}, rss=>"${root}gateway.cgi/changes/rss");
	} else {
		printHeader($$message{logo});
	}
	print "<p>$$message{license}</p>\n",
	      "<ul><li><em><a href='${root}list.cgi'>$$message{menu}</a></em></li>\n";
	print "<li><a href='http://shingetsu.sourceforge.net/'>",
	      "$$message{site}</a></li>\n",
	      "<li><a href='$ENV{SCRIPT_NAME}/motd'>$$message{agreement}</a></li></ul>",
	      "</body></html>\n";
}

#
# print MOTD file
#
sub printMotd() {
	print "Content-type: text/plain; charset=euc-jp\r\n",
	      "X-Shingetsu: shinGETsu/$Shingetsu::Config::version\r\n\r\n";
	open IN, $Shingetsu::Config::motd;
	while (<IN>) {
		print;
	}
	close IN;
}

#
# make list for RSS
#
sub makeRSS(%) {
	my %stamp = @_;
	local $_;
	my $now  = time;
	my $host = "http://$ENV{HTTP_HOST}";
	my $root = $Shingetsu::Config::root;
	my @list = ();
	foreach (keys %stamp) {
		if (($Shingetsu::Config::rssoutput>=0)
			&& ($stamp{$_} + $Shingetsu::Config::rssoutput*60*60 < $now)) {
			next;
		} elsif (/^(list|thread|note)_/) {
			my %x = ();
			$x{type}  = $1;
			$x{title} = file_decode($_);
			$x{str}	  = str_encode($x{title});
			$x{uri}	  = "$host$root$x{type}.cgi/$x{str}";
			$x{date}  = Shingetsu::Util::w3c_time($stamp{$_});
			push @list, \%x;
		}
	}
	return @list;
}

#
# print RSS
#
sub printRSS(%) {
	my %stamp = @_;
	my @list = makeRSS(%stamp);
	my $path_rss  = "http://$ENV{HTTP_HOST}$ENV{REQUEST_URI}";
	my $path_root = "http://$ENV{HTTP_HOST}$Shingetsu::Config::root";
	local $_;
#	print "Content-Type: text/xml\r\n\r\n";
	print "Content-Type: application/rdf+xml\r\n\r\n";
	print "<?xml version='1.0' encoding='utf-8'?>\n" .
	      "<rdf:RDF\n" .
	      "  xmlns='http://purl.org/rss/1.0/'\n" .
	      "  xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#'\n" .
	      "  xmlns:dc='http://purl.org/dc/elements/1.1/'\n" .
	      "  xml:lang='ja'>\n" .
	      "<channel rdf:about='$path_rss'>\n" .
	      "  <title>$$message{logo}</title>\n" .
	      "  <link>$path_root</link>\n" .
	      "  <description>$$message{logo}</description>\n" .
	      "  <items><rdf:Seq>\n";
	foreach (@list) {
		print "    <rdf:li rdf:resource='$$_{uri}' />\n";
	}
	print "  </rdf:Seq></items>\n</channel>\n";
	foreach (@list) {
                print "<item rdf:about='$$_{uri}'>\n" .
		      "  <title>$$_{title}</title>\n" .
		      "  <link>$$_{uri}</link>\n" .
		      "  <dc:date>$$_{date}</dc:date>\n" .
		      "</item>\n";
	}
	print  "</rdf:RDF>\n";
}

#
# print index page
#
sub printIndex() {
	printHeader($$message{index}, rss=>"/gateway.cgi/changes/rss");
	print "<form method='get' action='$ENV{SCRIPT_NAME}/delete'>\n";
	local $_;
	my $type;
	my %stat = Shingetsu::CacheStat::list();
	foreach $type (@types) {
		print "<h2>$$message{$type}</h2><ul>\n";
		foreach (sort glob("$datadir/${type}_*.dat")) {
			$_ =~ s|^$datadir/||;	$_ =~ s|.dat$||;
			my $x = file_decode($_);
			my $y = str_encode($x);
			my $records = $stat{$_}{records};
			my $size    = int($stat{$_}{size}/1024/1024*10)/10;
			print "<li><input type='radio' name='file' value='$_' tabindex='1' accesskey='s' />",
			      "<a href='${root}$type.cgi/$y'>$x</a>($records/$size$$message{mb})</li>\n";
		}
		print "</ul>"
	}
	print "<p><input type='submit' value='$$message{del_file}' tabindex='2' accesskey='d' /></p></form>";
	newElementForm();
	print "</body></html>\n";
}

#
# print recent changes
#
sub printChanges() {
	printHeader($$message{changes}, rss=>"/gateway.cgi/changes/rss");
	print "<form method='get' action='$ENV{SCRIPT_NAME}/delete'>\n";
	local $_;
	my $type;
	my %stat = Shingetsu::CacheStat::list();
	foreach $type (@types) {
		print "<h2>$$message{$type}</h2><ul>\n";
		my @buf = ();
		my %stamp = ();
		foreach (glob("$datadir/${type}_*.dat")) {
			$_ =~ s|^$datadir/||;	$_ =~ s|.dat$||;
			my $x = file_decode($_);
			my $y = str_encode($x);
			my $date = xlocaltime($stat{$_}{stamp});
			my $records = $stat{$_}{records};
			my $size    = int($stat{$_}{size}/1024/1024*10)/10;
			my $buf = "<li><input type='radio' name='file' value='$_' tabindex='1' accesskey='s' />$date <a href='${root}$type.cgi/$y'>$x</a>($records/$size$$message{mb})</li>\n";
			$stamp{$buf} = $stat{$_}{stamp};
			push @buf, "$buf";
		}
		@buf = sort {$stamp{$b} <=> $stamp{$a}} @buf;
		print @buf;
		print "</ul>"
	}
	print "<p><input type='submit' value='$$message{del_file}' tabindex='2' accesskey='d' /></p></form>";
	newElementForm();
	print "</body></html>\n";
}

#
# print recent changes in RSS form
#
sub printChangesRSS() {
	my %stat = Shingetsu::CacheStat::list();
	my %stamp = ();
	local $_;
	foreach (keys %stat) {
		$stamp{$_} = $stat{$_}{stamp};
	}
	printRSS(%stamp);
}

#
# print recent update
#
sub printUpdate() {
	local $_;
	my %stamp = ();
	if (open IN, $Shingetsu::Config::updatelist) {
		while (<IN>) {
			chomp;
			@_ = split /<>/;
			if ($_[2] =~ /^(list|thread|note)_/) {
				$stamp{$_[2]} = $_[0];
			}
		}
		close IN;
	} else {
		print404($$message{update});
		exit;
	}
	printHeader($$message{update}, rss=>"/gateway.cgi/update/rss");
	my $type;
	my %stat = Shingetsu::CacheStat::list();
	foreach $type (@types) {
		print "<h2>$$message{$type}</h2><ul>\n";
		my @buf = ();
		foreach (sort {$stamp{$b}<=>$stamp{$a}} grep(/^${type}_/, keys(%stamp))) {
			my $x = file_decode($_);
			my $y = str_encode($x);
			my $date = xlocaltime $stamp{$_};
			my $records = (defined $stat{$_})? $stat{$_}{records}: "?";
			my $size    = (defined $stat{$_})? int($stat{$_}{size}/1024/1024*10)/10: "?";
			print "<li>$date <a href='${root}$type.cgi/$y'>$x</a>($records/$size$$message{mb})</li>\n";
		}
		print "</ul>"
	}
	newElementForm();
	print "</body></html>\n";
}

#
# print recent update in RSS form
#
sub printUpdateRSS() {
	local $_;
	my %stamp = ();
	if (open IN, $Shingetsu::Config::updatelist) {
		while (<IN>) {
			chomp;
			@_ = split /<>/;
			if ($_[2] =~ /^(list|thread|note)_/) {
				$stamp{$_[2]} = $_[0];
			}
		}
		close IN;
	} else {
		print404($$message{update});
		exit;
	}
	printRSS(%stamp);
}

#
# search form
#
sub searchForm(;$) {
	my $query = (@_)? $_[0]: "";
	my $xquery = $query;
	$xquery =~ s/"/&quot;/g;
	print "<form method='get' action='$ENV{SCRIPT_NAME}/search'><p>",
	      "<input name='query' size='19' value=\"$xquery\" tabindex='1' accesskey='q' />",
	      "<input type='submit' value='$$message{search}' name='submit' tabindex='1' accesskey='s' />",
	      "</p></form>";

	my $gquery = $query;
#	$gquery =~ s/\|/ OR /g;
#	$gquery =~ s/\.[+*]?/ /g;
	$gquery =~ s/"/&quot;/g;
	print "<form method='get' action='$Shingetsu::Config::google'><p>",
	      "<input name='q' size='19' value=\"$gquery\" tabindex='1' accesskey='g' />",
	      "<input type='hidden' name='ie' value='UTF-8' />",
	      "<input type='submit' name='google' value='$$message{google}' tabindex='1' accesskey='g' />",
	      "</p></form>";

}

#
# search result
#
sub printSearchResult($) {
	my($query) = @_;
	printHeader($$message{search});
	print "\n";
	local $_;
	my($type, $file);
	my %stat = Shingetsu::CacheStat::list();
	foreach $type (@types) {
		print "<h2>$$message{$type}</h2><ul>\n";
		my @file = search($type, $query);
		foreach $file (@file) {
			my $x = file_decode($file);
			my $y = str_encode($x);
			my $records = $stat{$file}{records};
			my $size    = int($stat{$file}{size}/1024/1024*10)/10;
			print "<li><a href='${root}$type.cgi/$y'>$x</a>($records/$size$$message{mb})</li>\n";
		}
		print "</ul>"
	}
	searchForm($query);
	print "</body></html>";
}

#------------------------------------------------------------------#

#
# main routine
#
my ($path, $input) = readQuery();
my %arg = args($input);
my $cmd = ($arg{cmd})? $arg{cmd}: "";

if ($cmd eq "new") {
	xdie $$message{null_name} if ($arg{name} eq "");
	xdie $$message{bad_name}  if ($arg{name} =~ m|/|);
	xdie $$message{bad_name}  if ($arg{name} =~ m|\]\]|);
	xdie $$message{null_type} if ($arg{type} eq "");
	if ($arg{type} =~ /^(list|thread|note)$/) {
		print302("${root}$arg{type}.cgi/" . str_encode($arg{name}));
	} else {
		print404();
	}

} elsif ($path eq "delete") {
	checkAdmin();
	deleteDialog(%arg);

} elsif ($cmd eq "xdelete") {
	if (! defined $arg{file}) {
		print404();
		exit;
	} elsif ((defined $arg{stamp})&&(defined $arg{id})) {
		deleteRecord($arg{file}, $arg{stamp}, $arg{id}, \%arg);
	} else {
		deleteFile($arg{file});
	}

} elsif (($path eq "")||($path eq "title")) {
	printTitle();

} elsif ($path eq "list") {
	print302("${root}list.cgi");

} elsif ($path eq "motd") {
	printMotd();

} elsif ($path =~ m{^(list|thread|note)/([^/]+)$}) {
	print302("${root}$1.cgi/" . str_encode($2));

} elsif ($path eq "new") {
	printHeader("$$message{new}");
	newElementForm();
	print "</body></html>\n";

} elsif ($path eq "index") {
	checkAdmin();
	printIndex();

} elsif ($path eq "changes") {
	checkAdmin();
	printChanges();

} elsif ($path eq "update") {
	checkAdmin();
	printUpdate();

} elsif ($path eq "changes/rss") {
	checkAdmin();
	printChangesRSS();

} elsif ($path eq "update/rss") {
	checkAdmin();
	printUpdateRSS();

} elsif ($path =~ /^search/) {
	my $query = $';
	checkAdmin();
	if ((defined $query) && ($query =~ m|^/(.+)|)) {
		$query = $1;
		printSearchResult($query);
	} elsif ((defined $input) && ($input ne "")) {
		%arg = args($input);
		printSearchResult($arg{query});
	} else {
		printHeader($$message{search});
		searchForm();
		print "</body></html>";
	}

} elsif (($path eq "edittrust")
		&& ($ENV{HTTP_REFERER} =~ m{^http://[^/]+/(list|thread|note)\.cgi\?cmd=check})) {
	checkAdmin();
	%arg = args($input);
	if (! defined $arg{trip}) {
		print404();
		exit;
	} elsif ($arg{button} eq $$message{add}) {
		Shingetsu::Signature::addlist($arg{trip});
	} elsif ($arg{button} eq $$message{delete}) {
		Shingetsu::Signature::dellist($arg{trip});
	}
	if ($arg{type} =~ /^(list|thread|note)$/) {
		print302("${root}$arg{type}.cgi/" . str_encode(file_decode("$arg{type}_$arg{file}")));
	}

} else {
	print404();

}
